@@ -13,8 +13,10 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 #include "lua_common.h"
 #include "message.h"
+#include "libstat/stat_api.h"
 
 /* Textpart methods */
 /***
@@ -125,6 +127,14 @@ LUA_FUNCTION_DEF (textpart, get_stats);
  * @return {integer} number of words in the part
  */
 LUA_FUNCTION_DEF (textpart, get_words_count);
+
+/***
+ * @method mime_part:get_words()
+ * Get words in the part
+ * @return {table/strings} words in the part
+ */
+LUA_FUNCTION_DEF (textpart, get_words);
+
 /***
  * @method text_part:is_empty()
  * Returns `true` if the specified part is empty
@@ -168,6 +178,7 @@ static const struct luaL_reg textpartlib_m[] = {
 	LUA_INTERFACE_DEF (textpart, get_urls_length),
 	LUA_INTERFACE_DEF (textpart, get_lines_count),
 	LUA_INTERFACE_DEF (textpart, get_words_count),
+	LUA_INTERFACE_DEF (textpart, get_words),
 	LUA_INTERFACE_DEF (textpart, is_empty),
 	LUA_INTERFACE_DEF (textpart, is_html),
 	LUA_INTERFACE_DEF (textpart, get_html),
@@ -260,6 +271,13 @@ LUA_FUNCTION_DEF (mimepart, get_length);
  */
 LUA_FUNCTION_DEF (mimepart, get_type);
 
+/***
+ * @method mime_part:get_type_full()
+ * Extract content-type string of the mime part with all attributes
+ * @return {string,string,table} content type in form 'type','subtype', {attrs}
+ */
+LUA_FUNCTION_DEF (mimepart, get_type_full);
+
 /***
  * @method mime_part:get_cte()
  * Extract content-transfer-encoding for a part
@@ -355,6 +373,7 @@ static const struct luaL_reg mimepartlib_m[] = {
 	LUA_INTERFACE_DEF (mimepart, get_content),
 	LUA_INTERFACE_DEF (mimepart, get_length),
 	LUA_INTERFACE_DEF (mimepart, get_type),
+	LUA_INTERFACE_DEF (mimepart, get_type_full),
 	LUA_INTERFACE_DEF (mimepart, get_cte),
 	LUA_INTERFACE_DEF (mimepart, get_filename),
 	LUA_INTERFACE_DEF (mimepart, get_header),
@@ -644,6 +663,34 @@ lua_textpart_get_words_count (lua_State *L)
 	return 1;
 }
 
+static gint
+lua_textpart_get_words (lua_State *L)
+{
+	struct rspamd_mime_text_part *part = lua_check_textpart (L);
+	rspamd_stat_token_t *w;
+	guint i;
+
+	if (part == NULL) {
+		return luaL_error (L, "invalid arguments");
+	}
+
+	if (IS_PART_EMPTY (part) || part->normalized_words == NULL) {
+		lua_createtable (L, 0, 0);
+	}
+	else {
+		lua_createtable (L, part->normalized_words->len, 0);
+
+		for (i = 0; i < part->normalized_words->len; i ++) {
+			w = &g_array_index (part->normalized_words, rspamd_stat_token_t, i);
+
+			lua_pushlstring (L, w->begin, w->len);
+			lua_rawseti (L, -2, i + 1);
+		}
+	}
+
+	return 1;
+}
+
 static gint
 lua_textpart_is_empty (lua_State * L)
 {
@@ -815,9 +862,12 @@ lua_mimepart_get_length (lua_State * L)
 }
 
 static gint
-lua_mimepart_get_type (lua_State * L)
+lua_mimepart_get_type_common (lua_State * L, gboolean full)
 {
 	struct rspamd_mime_part *part = lua_check_mimepart (L);
+	GHashTableIter it;
+	gpointer k, v;
+	struct rspamd_content_type_param *param;
 
 	if (part == NULL) {
 		lua_pushnil (L);
@@ -828,7 +878,53 @@ lua_mimepart_get_type (lua_State * L)
 	lua_pushlstring (L, part->ct->type.begin, part->ct->type.len);
 	lua_pushlstring (L, part->ct->subtype.begin, part->ct->subtype.len);
 
-	return 2;
+	if (!full) {
+		return 2;
+	}
+
+	lua_createtable (L, 0, 2 + (part->ct->attrs ?
+			g_hash_table_size (part->ct->attrs) : 0));
+
+	if (part->ct->charset.len > 0) {
+		lua_pushstring (L, "charset");
+		lua_pushlstring (L, part->ct->charset.begin, part->ct->charset.len);
+		lua_settable (L, -3);
+	}
+
+	if (part->ct->boundary.len > 0) {
+		lua_pushstring (L, "charset");
+		lua_pushlstring (L, part->ct->boundary.begin, part->ct->boundary.len);
+		lua_settable (L, -3);
+	}
+
+	if (part->ct->attrs) {
+		g_hash_table_iter_init (&it, part->ct->attrs);
+
+		while (g_hash_table_iter_next (&it, &k, &v)) {
+			param = v;
+
+			if (param->name.len > 0 && param->name.len > 0) {
+				/* TODO: think about multiple values here */
+				lua_pushlstring (L, param->name.begin, param->name.len);
+				lua_pushlstring (L, param->value.begin, param->value.len);
+				lua_settable (L, -3);
+			}
+		}
+	}
+
+	return 3;
+}
+
+static gint
+lua_mimepart_get_type (lua_State * L)
+{
+	return lua_mimepart_get_type_common (L, FALSE);
+}
+
+static gint
+lua_mimepart_get_type_full (lua_State * L)
+{
+	return lua_mimepart_get_type_common (L, TRUE);
 }
 
 static gint
