@@ -625,7 +625,6 @@ if opts['reporting'] == true then
   elseif not opts['send_reports'] then
     dmarc_reporting = true
     rspamd_config:add_on_load(function(cfg, ev_base, worker)
-      if not (worker:get_name() == 'normal' and worker:get_index() == 0) then return end
       load_scripts(cfg, ev_base)
     end)
   else
@@ -642,9 +641,9 @@ if opts['reporting'] == true then
       end
     end
     rspamd_config:add_on_load(function(cfg, ev_base, worker)
-      if not (worker:get_name() == 'normal' and worker:get_index() == 0) then return end
-      local rresolver = rspamd_resolver.init(ev_base, rspamd_config)
       load_scripts(cfg, ev_base)
+      if not (worker:get_name() == 'controller' and worker:get_index() == 0) then return end
+      local rresolver = rspamd_resolver.init(ev_base, rspamd_config)
       rspamd_config:register_finish_script(function ()
         local stamp = pool:get_variable(VAR_NAME, 'double')
         if not stamp then
